%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%	GPIB_TRIG_FRAME_DEL.m
%
% 	Routine for remote control of Tektronik Scope TDS 784D
%	Required to measure the delay between 2pps and CDMA2000 Forward link
%	frame begin
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

close all;

TRIG_FRAME_DEL_PAR;                                                 %Loading of parameters

instrreset;                                                         %disconnect and delete all instrument objects

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Define the number of waveform record points that we want from the scope.
% This needs to be determined early so that we can prepare the buffers for
% the GPIB interface of the scope.
%
nof_entire_samples=50000;                                           % capture samples by the scope
tds_wfm_pt_size=5;                                                  % maximum size (in chars) of a wfm point.
                                                                    % (used only for buffer calculations
tds_block_size=0;                                                   % Set the number of wfm points to read at one time
                                                                    % (If set to zero, the entire waveform will be read.)
                                                                  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Calculate the block sizes to use.
%
if tds_block_size==0
    block_size = nof_entire_samples;
else
    block_size = tds_block_size;
end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
instrreset;                                                         %disconnect and delete all instrument objects
tds = gpib('ni',GPIB_BORD_INDEX,TDS_PRIM);
set(tds,'InputBufferSize', block_size * tds_wfm_pt_size);           %Set buffer size for input and output
set(tds,'OutputBufferSize',10000);
set(tds,'Timeout',100);                                             %Set GPIB timeout to 100 seconds

cmu_base = gpib('ni',GPIB_BORD_INDEX,CMU_PRIM,'SECOND',BASE_CMU);
set(cmu_base,'InputBufferSize',20000);                              %Set buffer size for input and output
set(cmu_base,'OutputBufferSize',10000);

cmu_cdma = gpib('ni',GPIB_BORD_INDEX,CMU_PRIM,'SECOND',CDMA800_NSIG);
set(cmu_cdma,'InputBufferSize',20000);                              %Set buffer size for input and output
set(cmu_cdma,'OutputBufferSize',10000);

fopen(tds);                                                         %Open device channel TDS
fopen(cmu_base);                                                    %Open device channel CMU base
fopen(cmu_cdma);                                                    %Open device channel CMU CDMA

disp(['Data collection starting']);

fprintf(tds, '*RST');                                               %Reset TDS
fprintf(tds, '*IDN?');
IDN = fscanf(tds);
fprintf(tds, '*OPT?');
OPT = fscanf(tds);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

eval(['load ', data_path, 'mixer_LP.mat']);
mixer_LP_taps=Num;                                                  %mixer lowpass
mixer_LP_del=(length(mixer_LP_taps)-1)/2;
mixer_LP_len=length(mixer_LP_taps);                                 %Length of mixer LP

eval(['load ', data_path, 'h_is95_equ.mat']);
h_equ=[h_e_neg h_e_pos];
h_equ_len=length(h_equ);                                            %Length of inverse Equalizer filter

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Calculation ofr pre and post samples
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
dec_rate=f_sample_scope/(f_chip*ov_eval);
entire_postproc_settling_samples=mixer_LP_len+dec_rate*h_equ_len;   %settling length im post processing
trig_uncertainty_samples=trigger_uncertainty_symb*dec_rate*ov_eval;

nof_pre_samples=trig_uncertainty_samples+entire_postproc_settling_samples;
nof_post_samples=nof_post_symb*dec_rate*ov_eval+trig_uncertainty_samples;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% (jdb) nof_entire_samples=50000;                                           % capture samples by the scope
nof_samples_RF=round(nof_pre_samples+nof_post_samples);             % nof RF samples to be read

trigger_pos=round(nof_pre_samples/nof_entire_samples*100);

nof_samples_trig=round((trigger_pos/100)*nof_entire_samples*2);     % nof trig samples to be read


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Grundparameter fr CMU200, CDMA einstellen
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fprintf(cmu_base, 'SYST:REM:ADDR:SEC 1,"CDMA2KCellMS_NSig"');       %secondary address CDMA NSIG

fprintf(cmu_cdma, 'SOURce:RFGenerator:FREQuency:UNIT MHZ');         %frequency unit MHz
fprintf(cmu_cdma, 'OUTP:STAT RF2');                                 %RF Connector 2
fprintf(cmu_cdma, 'SENS:CORR:LOSS:OUTP2 0.0');                      %No external loss

fprintf(cmu_cdma, 'SOURce:RFGenerator:POWer:CDMA -16dBm');          %Output level -16dBm
fprintf(cmu_cdma, 'SOURce:RFGenerator:LEVel:FPICh -7');             %Pilot -7dB
fprintf(cmu_cdma, 'SOURce:RFGenerator:LEVel:FFCH OFF');             %F-FCH OFF
fprintf(cmu_cdma, 'SOURce:RFGenerator:LEVel:FSYNc OFF');            %F-SYNC OFF
fprintf(cmu_cdma, 'SOURce:RFGenerator:LEVel:FPCH OFF');             %F-PCH OFF

fprintf(cmu_cdma, 'SOURce:RFGenerator:LEVel:OCNS:ENABle OFF');      %OCNS OFF
fprintf(cmu_cdma, 'SOURce:RFGenerator:FREQuency 50');               %Tx freq 50MHz
fprintf(cmu_cdma, 'SOURce:RFGenerator:PROPerty:PNOFfset 0');        %PN Offset 0
fprintf(cmu_cdma, 'INIT:RFGENERATOR');                              %Switch On generator

disp('..CMU Outputs enabled');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Configure parameters for TDS 784D
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf(tds, '*CLS');                                               %clear event status register
fprintf(tds, 'HORIZONTAL:MAIN:SCALE 200e-9');                       %200ns/div -> 250Ms/s
fprintf(tds, ['HORIZONTAL:TRIGGER:POSITION ', num2str(trigger_pos)]); %trigger position

fprintf(tds, 'SELECT:CH1 ON');                                      %activate channel 1
fprintf(tds, 'CH1:BANDWIDTH FULL');                                 %bandwidth
fprintf(tds, 'CH1:COUPLING DC');                                    %coupling
fprintf(tds, 'CH1:IMPEDANCE FIFTY');                                %impedance
fprintf(tds, 'CH1:OFFSET 0');                                       %offset
fprintf(tds, 'CH1:POSITION 0');                                     %position
fprintf(tds, 'CH1:SCALE 0.02');                                     %scale (for -21dBm rms)

fprintf(tds, 'SELECT:CH2 ON');                                      %activate channel 2
fprintf(tds, 'CH2:BANDWIDTH FULL');                                 %bandwidth
fprintf(tds, 'CH2:COUPLING DC');                                    %coupling
fprintf(tds, 'CH2:IMPEDANCE MEG');                                  %impedance
fprintf(tds, 'CH2:OFFSET 0');                                       %offset
fprintf(tds, 'CH2:POSITION 0');                                     %position
fprintf(tds, 'CH2:SCALE 2');                                        %scale

fprintf(tds, 'TRIGGER:MAIN:EDGE:SOURCE CH2');
fprintf(tds, 'TRIGGER:MAIN:EDGE:SLOPE RISE');
fprintf(tds, 'TRIGGER:MAIN:LEVEL 0.5');

fprintf(tds, 'ACQUIRE:REPET OFF');
fprintf(tds, 'ACQUIRE:STOPAFTER SEQUENCE');

fprintf(tds, '*ESR?');                                              %get event status register
SESR=fscanf(tds);
if str2num(SESR) ~= 0
    error(['There was an error, SESR = ',SESR]);
end;

disp('..Scope configured');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Start acquisition
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf(tds, ['HORIZONTAL:RECORDLENGTH ', num2str(nof_entire_samples)]);
fprintf(tds, 'HORIZONTAL:RECORDLENGTH?');
record_length_str=fscanf(tds);
record_length=GPIB_convert(record_length_str,1);
if record_length~=nof_entire_samples
    error('nof_samples is not possible on scope, please select appropriate nof_samples');
end;
fprintf(tds, 'ACQUIRE:STATE RUN');                                  %start aquisition

disp('..Scope has been armed');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Wait for the trigger to occur (this seems to be needed for the TDS540A scope).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
trigger_state_buf = 'empty';
while ~strcmp(trigger_state_buf,'SAV')
    pause(0.1);
    fprintf(tds, 'TRIG:STATE?');
    trigger_state_buf = fscanf(tds,'%s');
    disp(['...Current trigger state: ',trigger_state_buf]);
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Read Ch1 buffers (RF Signal)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf(tds, 'DATA:SOURCE CH1');
fprintf(tds, 'DATA:ENCDG ASCI');
fprintf(tds, 'DATA:WIDTH 1');

signal_RF=zeros(1,nof_samples_RF);
signal_str=zeros(1,block_size * tds_wfm_pt_size);
if tds_block_size~=0
    time_sum=0;
    for i=1:block_size:nof_samples_RF
        tic;
        start=i;
        stop=min(i+block_size-1,nof_samples_RF);
        disp(['Reading RF samples ',num2str(start),'-',num2str(stop),' of ', num2str(nof_samples_RF), ' from scope']);
        fprintf(tds, ['DATA:START ', num2str(start)]);
        fprintf(tds, ['DATA:STOP ', num2str(stop)]);
        fprintf(tds, 'WFMPRE?');
        header_str=fscanf(tds);
        fprintf(tds, 'CURVE?');
        signal_str=fscanf(tds);
        signal_RF(start:stop)=GPIB_convert(signal_str,(1:stop-start+1));            %convert block to numbers and save
        t=toc;
        time_sum=time_sum+t;
        disp([' Read took ',num2str(time_sum),' of an expected ', num2str(nof_samples_RF/block_size*t), ' seconds']);
    end;
else
    tic;
    start=1;
    stop=nof_samples_RF;
    disp(['Reading RF samples ',num2str(start),'-',num2str(stop),' from scope']);
    fprintf(tds, ['DATA:START ', num2str(start)]);
    fprintf(tds, ['DATA:STOP ', num2str(stop)]);
    fprintf(tds, 'WFMPRE?');
    header_str=fscanf(tds);
    fprintf(tds, 'CURVE?');
    signal_str=fscanf(tds);
    signal_RF(start:stop)=GPIB_convert(signal_str,(1:stop-start+1));            %convert block to numbers and save
    t=toc;
    disp([' Read took ',num2str(t),' seconds']);
end;




fprintf(tds, 'WFMPRE:CH1:XINCR?');
sampling_periode_str=fscanf(tds);
sampling_periode=GPIB_convert(sampling_periode_str,1);
if (1/f_sample_scope)~=sampling_periode
    error('Scopes samples not with desired rate');
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Read Ch2 buffers (Trigger Signal)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf(tds, 'DATA:SOURCE CH2');
fprintf(tds, 'DATA:ENCDG ASCI');
fprintf(tds, 'DATA:WIDTH 1');

signal_trig=zeros(1,nof_samples_RF);
signal_str=zeros(1,block_size * tds_wfm_pt_size);
if tds_block_size~=0
    time_sum=0;
    for i=1:block_size:nof_samples_trig
        tic;
        start=i;
        stop=min(i+block_size-1,nof_samples_trig);
        disp(['Reading Trigger samples ',num2str(start),'-',num2str(stop),' of ', num2str(nof_samples_trig), ' from scope']);
        fprintf(tds, ['DATA:START ', num2str(start)]);
        fprintf(tds, ['DATA:STOP ', num2str(stop)]);
        fprintf(tds, 'WFMPRE?');
        header_str=fscanf(tds);
        fprintf(tds, 'CURVE?');
        signal_str=fscanf(tds);
        signal_trig(start:stop)=GPIB_convert(signal_str,(1:stop-start+1));        %convert block to numbers and save
        t=toc;
        time_sum=time_sum+t;
        disp([' Read took ',num2str(time_sum),' of an expected ', num2str(nof_samples_trig/block_size*t), ' seconds']);
    end;
else
    tic;
    start=1;
    stop=nof_samples_trig;
    disp(['Reading Trigger samples ',num2str(start),'-',num2str(stop),' from scope']);
    fprintf(tds, ['DATA:START ', num2str(start)]);
    fprintf(tds, ['DATA:STOP ', num2str(stop)]);
    fprintf(tds, 'WFMPRE?');
    header_str=fscanf(tds);
    fprintf(tds, 'CURVE?');
    signal_str=fscanf(tds);
    signal_trig(start:stop)=GPIB_convert(signal_str,(1:stop-start+1));            %convert block to numbers and save
    t=toc;
    disp([' Read took ',num2str(t),' seconds']);
end;

eval(['save ', data_path, 'GPIB_signal signal_RF signal_trig']);

disp(['Data collection completed.']);

fclose(tds);                                                        %Close device channel TDS
fclose(cmu_base);                                                   %Close device channel CMU base
fclose(cmu_cdma);                                                   %Close device channel CMU CDMA


